/*
 * $Id: tests.c,v 1.6 2004/09/20 02:42:53 anholt Exp $
 *
 * Copyright © 2004 Eric Anholt
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Eric Anholt not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Eric Anholt makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * ERIC ANHOLT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL ERIC ANHOLT BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include "rendercheck.h"

extern int win_width, win_height;
extern int pixmap_move_iter;
extern Bool is_verbose;

color4d colors[] = {
	{1.0, 1.0, 1.0, 1.0},
	{1.0, 0, 0, 1.0},
	{0, 1.0, 0, 1.0},
	{0, 0, 1.0, 1.0},
	{0.5, 0, 0, 0.25},
	{0.5, 0, 0, .5},
	{0.0, .5, 1.0, .5},
	{0.0, .5, 1.0, 0}
};

int num_colors = sizeof(colors) / sizeof(colors[0]);

struct op_info {
	int op;
	char *name;
} ops[] = {
	{PictOpClear, "Clear"},
	{PictOpSrc, "Src"},
	{PictOpDst, "Dst"},
	{PictOpOver, "Over"},
	{PictOpOverReverse, "OverReverse"},
	{PictOpIn, "In"},
	{PictOpInReverse, "InReverse"},
	{PictOpOut, "Out"},
	{PictOpOutReverse, "OutReverse"},
	{PictOpAtop, "Atop"},
	{PictOpAtopReverse, "AtopReverse"},
	{PictOpXor, "Xor"},
	{PictOpAdd, "Add"},
	{PictOpSaturate, "Saturate"},
	{PictOpDisjointClear, "DisjointClear"},
	{PictOpDisjointSrc, "DisjointSrc"},
	{PictOpDisjointDst, "DisjointDst"},
	{PictOpDisjointOver, "DisjointOver"},
	{PictOpDisjointOverReverse, "DisjointOverReverse"},
	{PictOpDisjointIn, "DisjointIn"},
	{PictOpDisjointInReverse, "DisjointInReverse"},
	{PictOpDisjointOut, "DisjointOut"},
	{PictOpDisjointOutReverse, "DisjointOutReverse"},
	{PictOpDisjointAtop, "DisjointAtop"},
	{PictOpDisjointAtopReverse, "DisjointAtopReverse"},
	{PictOpDisjointXor, "DisjointXor"},
	{PictOpConjointClear, "ConjointClear"},
	{PictOpConjointSrc, "ConjointSrc"},
	{PictOpConjointDst, "ConjointDst"},
	{PictOpConjointOver, "ConjointOver"},
	{PictOpConjointOverReverse, "ConjointOverReverse"},
	{PictOpConjointIn, "ConjointIn"},
	{PictOpConjointInReverse, "ConjointInReverse"},
	{PictOpConjointOut, "ConjointOut"},
	{PictOpConjointOutReverse, "ConjointOutReverse"},
	{PictOpConjointAtop, "ConjointAtop"},
	{PictOpConjointAtopReverse, "ConjointAtopReverse"},
	{PictOpConjointXor, "ConjointXor"},
};

int num_ops = sizeof(ops) / sizeof(ops[0]);




void
begin_test(Display *dpy, picture_info *win)
{
	int i, j, src, dst, mask;
	int num_dests, num_formats;
	picture_info *picture_40x40;
	picture_info *mask_20x20;
	picture_info *picture_1x1;
	XRenderColor rendercolor;
	XRenderPictureAttributes pa;

	num_formats=3;

//40x40 picture 
	picture_40x40 = (picture_info *)malloc(sizeof(picture_info));
	if (picture_40x40 == NULL)
		errx(1, "malloc error");
		i=7;
//PictStandardARGB32 0
//PictStandardRGB24  1
//PictStandardA8     2
//PictStandardA4     3
//PictStandardA1     4
//PictStandardNUM    5

	picture_40x40[0].format = XRenderFindStandardFormat(dpy,
	    i % num_formats);
	picture_40x40[0].d = XCreatePixmap(dpy, RootWindow(dpy, 0), 40,
	    40, picture_40x40[0].format->depth);
	picture_40x40[0].pict = XRenderCreatePicture(dpy,
	    picture_40x40[0].d, picture_40x40[0].format, 0, NULL);

	picture_40x40[0].name = (char *)malloc(20);
	if (picture_40x40[0].name == NULL)
		errx(1, "malloc error");
	sprintf(picture_40x40[0].name, "40x40 ");
	describe_format(picture_40x40[0].name +
	    strlen(picture_40x40[0].name), 20 -
	    strlen(picture_40x40[0].name), picture_40x40[0].format);

	rendercolor.red = colors[i / num_formats].r * 65535;
	rendercolor.green = colors[i / num_formats].g * 65535;
	rendercolor.blue = colors[i / num_formats].b * 65535;
	//rendercolor.alpha = colors[i / num_formats].a * 65535;
	rendercolor.alpha = 0;
	XRenderFillRectangle(dpy, PictOpSrc, picture_40x40[0].pict,
	    &rendercolor, 0, 0, 40, 40);

//1x1 picture
	picture_1x1 = (picture_info *)malloc(sizeof(picture_info));
	if (picture_1x1 == NULL)
		errx(1, "malloc error");
		i=7;
	picture_1x1[0].format = XRenderFindStandardFormat(dpy,
	    i % num_formats);
	picture_1x1[0].d = XCreatePixmap(dpy, RootWindow(dpy, 0), 1,
	    1, picture_1x1[0].format->depth);
	pa.repeat = TRUE;
//with repeat bit open
	picture_1x1[0].pict = XRenderCreatePicture(dpy,
	    picture_1x1[0].d, picture_1x1[0].format, CPRepeat, &pa);
//no repeat bit
//	picture_1x1[0].pict = XRenderCreatePicture(dpy,
//	    picture_1x1[0].d, picture_1x1[0].format, 0, NULL);

	picture_1x1[0].name = (char *)malloc(20);
	if (picture_1x1[0].name == NULL)
		errx(1, "malloc error");
	sprintf(picture_1x1[0].name, "1x1R ");
	describe_format(picture_1x1[0].name +
	    strlen(picture_1x1[0].name), 20 -
	    strlen(picture_1x1[0].name), picture_1x1[0].format);

	rendercolor.red = colors[i / num_formats].r * 65535;
	rendercolor.green = colors[i / num_formats].g * 65535;
	rendercolor.blue = colors[i / num_formats].b * 65535;
	//rendercolor.alpha = colors[i / num_formats].a * 65535;
	rendercolor.alpha = 0;
	XRenderFillRectangle(dpy, PictOpSrc, picture_1x1[0].pict,
	    &rendercolor, 0, 0, 1, 1);


//20x20 mask 
	mask_20x20 = (picture_info *)malloc(sizeof(picture_info));
	if(mask_20x20 == NULL)
		errx(1, "malloc error");
	mask_20x20[0].format = XRenderFindStandardFormat(dpy,
	    2);
	mask_20x20[0].d = XCreatePixmap(dpy, RootWindow(dpy, 0), 20,
	    20, mask_20x20[0].format->depth);
	mask_20x20[0].pict = XRenderCreatePicture(dpy,
	    mask_20x20[0].d, mask_20x20[0].format, 0, NULL);

	mask_20x20[0].name = (char *)malloc(20);
	if (mask_20x20[0].name == NULL)
		errx(1, "malloc error");
	sprintf(mask_20x20[0].name, "20x20 ");
	describe_format(mask_20x20[0].name +
	    strlen(mask_20x20[0].name), 20 -
	    strlen(mask_20x20[0].name), mask_20x20[0].format);
	rendercolor.red = 0;
	rendercolor.green = 0;
	rendercolor.blue = 0;
	rendercolor.alpha = 40000;
	XRenderFillRectangle(dpy, PictOpSrc, mask_20x20[0].pict,
	    &rendercolor, 0, 0, 20, 20);
	XFlush(dpy);

// XRenderComposite(Display *dpy,
// 			int  op,
// 			Picture src,
// 			Picture mask,
// 			Picture dsk,
// 			int src_x,
// 			int src_y,
// 			int mask_x,
// 			int mask_y,
// 			int dst_x,
// 			int dst_y,
// 			unsigned int width,
// 			unsigned int height)

//With mask picture 1x1
	XRenderComposite(dpy, PictOpSrc, picture_1x1[0].pict, mask_20x20[0].pict, win->pict, 0, 0, 5, 5, 50, 50, 40, 40);
//No mask picture 1x1
//	XRenderComposite(dpy, PictOpSrc, picture_1x1[0].pict, 0, win->pict, 0, 0, 0, 0, 50, 50, 40, 40);

	XFlush(dpy);

//	}
}
